import {Group, groupData} from './export/group';
import {User, userData}   from './export/user';
import colors from 'colors';

function test_wrapper(func: string, cond: boolean) {
    console.log(colors.blue("Test simpliciste de "+func+"..."));
    try {
        if (cond) console.log(colors.green("Test passé avec succès !"));
        else      console.log(colors.yellow("Erreur bizarre"));
    }
    catch {
        console.log(colors.red("Echec catastrophique !"));
        return;
    }
}

// Tests simples et naïfs sur les groupes seuls
async function simple_tests_Group() {
    // Données bidons
    let test_data = new groupData();
    test_data.gid = "test_group";
    test_data.name = "test_group";
    // Tests encapsulés
    test_wrapper("Group.search", (await Group.search(test_data)).length == 0);
    test_wrapper("Group.create", await Group.create(test_data) && (await Group.search(test_data))[0] == test_data.gid);
    test_wrapper("Group.peek", (await Group.peek(test_data.gid))['name'] == test_data.name);
    test_wrapper("Group.edit", await Group.edit(test_data) && (await Group.peek(test_data.gid))['name'] == "test_group2");
    test_wrapper("Group.delete", await Group.delete(test_data.gid) &&  (await Group.search(test_data)).length == 0);
}
// Tests simples et naïfs sur les utilisateurs seuls
async function simple_tests_User() {
    // Données bidons
    let test_data = new userData();
    test_data.uid = "test_given_name.last_name";
    test_data.givenName = "Test_given_name";
    test_data.lastName = "Last_name";
    // Tests encapsulés
    test_wrapper("User.search", (await User.search(test_data)).length == 0);
    test_wrapper("User.create", await User.create(test_data) && (await User.search(test_data))[0] == test_data.uid);
    test_wrapper("User.peek", (await User.peek(test_data.uid))['givenName'] == test_data.givenName);
    test_wrapper("User.edit", await User.edit(test_data) && (await User.peek(test_data.uid))['givenName'] == "Test_given_name2");
    test_wrapper("User.delete", await User.delete(test_data.uid) &&  (await User.search(test_data)).length == 0);
}
// Tests simples et naïfs sur les fonctions dites couplées ; ajout et retrait de membre
async function simple_coupled_tests() {
    let test_data_group  = new groupData();
    test_data_group.gid  = "test_group";
    test_data_group.name = "test_group";
    
    let test_data_user       = new userData();
    test_data_user.uid       = "test_given_name.last_name";
    test_data_user.givenName = "Test_given_name";
    test_data_user.lastName  = "Last_name";

    await Group.delete("test_group");
    await User.delete("test_given_name.last_name");
    // Create dummies
    await Group.create(test_data_group);
    await User.create(test_data_user);

    test_wrapper("Group.addAdmin", await Group.addAdmin(test_data_user.uid, test_data_group.gid));
                               //&& (await Group.peek(test_data_group.gid))['admins'] == [test_data_user.uid]
                               //&& (await Group.peek(test_data_user.uid)) ['admins'] == [test_data_group.gid]);
    /** test_wrapper("Group.remAdmin", await Group.remAdmin(test_data_user.uid, test_data_group.gid)
                               && (await Group.peek(test_data_group.gid))['admins'] == []
                               && (await Group.peek(test_data_user.uid)) ['admins'] == []);
    test_wrapper("Group.addSpeaker", await Group.addSpeaker(test_data_user.uid, test_data_group.gid)
                                 && (await Group.peek(test_data_group.gid))['speakers'] == [test_data_user.uid]
                                 && (await Group.peek(test_data_user.uid)) ['speakers'] == [test_data_group.gid]);
    test_wrapper("Group.remSpeaker", await Group.remSpeaker(test_data_user.uid, test_data_group.gid)
                                 && (await Group.peek(test_data_group.gid))['speakers'] == []
                                 && (await Group.peek(test_data_user.uid)) ['speakers'] == []);
    test_wrapper("Group.addMember", await Group.addMember(test_data_user.uid, test_data_group.gid)
                                && (await Group.peek(test_data_group.gid))['members'] == [test_data_user.uid]
                                && (await Group.peek(test_data_user.uid)) ['members'] == [test_data_group.gid]);
    test_wrapper("Group.remMember", await Group.remMember(test_data_user.uid, test_data_group.gid)
                                && (await Group.peek(test_data_group.gid))['members'] == []
                                && (await Group.peek(test_data_user.uid)) ['members'] == []);
    test_wrapper("Group.addFollower", await Group.addFollower(test_data_group.gid, test_data_user.uid)
                                  && (await Group.peek(test_data_group.gid))['followers'] == [test_data_user.uid]
                                  && (await Group.peek(test_data_user.uid)) ['followers'] == [test_data_group.gid]);
    test_wrapper("Group.remFollower", await Group.remFollower(test_data_group.gid, test_data_user.uid)
                                  && (await Group.peek(test_data_group.gid))['followers'] == []
                                  && (await Group.peek(test_data_user.uid)) ['followers'] == []);*/
    
    //Group.delete(test_data_group.gid);
    //User.delete(test_data_user.uid);
}

async function main() {
    //simple_tests_Group();
    //simple_tests_User();
    simple_coupled_tests();
}

main()